@@ -29,6 +29,7 @@ __all__ = [
     "AuthToken",
     "basic_auth",
     "kerberos_auth",
+    "bearer_auth",
     "custom_auth",
     "Bookmark",
     "ServerInfo",
@@ -59,6 +60,7 @@ __all__ = [
 from logging import getLogger
 
 
+from neo4j._deadline import Deadline
 from neo4j.addressing import (
     Address,
     IPv4Address,
@@ -69,6 +71,7 @@ from neo4j.api import (
     AuthToken,
     basic_auth,
     kerberos_auth,
+    bearer_auth,
     custom_auth,
     Bookmark,
     ServerInfo,
@@ -317,21 +320,28 @@ class Driver:
         Use the exception to further understand the cause of the connectivity problem.
 
         Note: Even if this method throws an exception, the driver still need to be closed via close() to free up all resources.
+
+        .. note::
+            \\*\\*config is experimental may change in the future.
+
         """
         raise NotImplementedError
 
-    @experimental("Feature support query, based on Bolt Protocol Version and Neo4j Server Version will change in the future.")
+    @experimental("Feature support query, based on Bolt protocol version and Neo4j server version will change in the future.")
     def supports_multi_db(self):
         """ Check if the server or cluster supports multi-databases.
 
         :return: Returns true if the server or cluster the driver connects to supports multi-databases, otherwise false.
         :rtype: bool
-        """
-        cx = self._pool.acquire(access_mode=READ_ACCESS, timeout=self._pool.workspace_config.connection_acquisition_timeout, database=self._pool.workspace_config.database)
-        support = cx.supports_multiple_databases
-        self._pool.release(cx)
 
-        return support
+
+        .. note::
+            Feature support query, based on Bolt Protocol Version and Neo4j
+            server version will change in the future.
+        """
+        with self.session() as session:
+            session._connect(READ_ACCESS)
+            return session._connection.supports_multiple_databases
 
 
 class BoltDriver(Direct, Driver):
@@ -435,23 +445,26 @@ class Neo4jDriver(Routing, Driver):
         return self._verify_routing_connectivity()
 
     def _verify_routing_connectivity(self):
-        from neo4j.exceptions import ServiceUnavailable
-        from neo4j._exceptions import BoltHandshakeError
+        from neo4j.exceptions import (
+            Neo4jError,
+            ServiceUnavailable,
+            SessionExpired,
+        )
 
         table = self._pool.get_routing_table_for_default_database()
+        timeout = self._default_workspace_config.connection_acquisition_timeout
         routing_info = {}
         for ix in list(table.routers):
             try:
                 routing_info[ix] = self._pool.fetch_routing_info(
                     address=table.routers[0],
                     database=self._default_workspace_config.database,
+                    imp_user=self._default_workspace_config.impersonated_user,
                     bookmarks=None,
-                    timeout=self._default_workspace_config
-                                .connection_acquisition_timeout
+                    deadline=Deadline(timeout)
                 )
-            except BoltHandshakeError as error:
+            except (ServiceUnavailable, SessionExpired, Neo4jError):
                 routing_info[ix] = None
-
         for key, val in routing_info.items():
             if val is not None:
                 return routing_info
