@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- encoding: utf-8 -*-
 
-# Copyright (c) 2002-2020 "Neo4j,"
+# Copyright (c) "Neo4j"
 # Neo4j Sweden AB [http://neo4j.com]
 #
 # This file is part of Neo4j.
@@ -185,6 +185,13 @@ class PoolConfig(Config):
     connection_timeout = 30.0  # seconds
     # The maximum amount of time to wait for a TCP connection to be established.
 
+    #: Update Routing Table Timout
+    update_routing_table_timeout = 90.0  # seconds
+    # The maximum amount of time to wait for updating the routing table.
+    # This includes everything necessary for this to happen.
+    # Including opening sockets, requesting and receiving the routing table,
+    # etc.
+
     #: Trust
     trust = TRUST_SYSTEM_CA_SIGNED_CERTIFICATES
     # Specify how to determine the authenticity of encryption certificates provided by the Neo4j instance on connection.
@@ -232,36 +239,23 @@ class PoolConfig(Config):
         # TLS 1.1 - Released in 2006, published as RFC 4346. (Disabled)
         # TLS 1.2 - Released in 2008, published as RFC 5246.
 
-        try:
-            # python 3.6+
-            # https://docs.python.org/3.6/library/ssl.html#ssl.PROTOCOL_TLS_CLIENT
-            ssl_context = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
+        # https://docs.python.org/3.6/library/ssl.html#ssl.PROTOCOL_TLS_CLIENT
+        ssl_context = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
 
-            # For recommended security options see
-            # https://docs.python.org/3.6/library/ssl.html#protocol-versions
-            ssl_context.options |= ssl.OP_NO_TLSv1      # Python 3.2
-            ssl_context.options |= ssl.OP_NO_TLSv1_1    # Python 3.4
+        # For recommended security options see
+        # https://docs.python.org/3.6/library/ssl.html#protocol-versions
+        ssl_context.options |= ssl.OP_NO_TLSv1      # Python 3.2
+        ssl_context.options |= ssl.OP_NO_TLSv1_1    # Python 3.4
 
-        except AttributeError:
-            # python 3.5
-            # https://docs.python.org/3.5/library/ssl.html#ssl.PROTOCOL_TLS
-            ssl_context = ssl.SSLContext(ssl.PROTOCOL_TLS)
-
-            # For recommended security options see
-            # https://docs.python.org/3.5/library/ssl.html#protocol-versions
-            ssl_context.options |= ssl.OP_NO_SSLv2      # Python 3.2
-            ssl_context.options |= ssl.OP_NO_SSLv3      # Python 3.2
-            ssl_context.options |= ssl.OP_NO_TLSv1      # Python 3.2
-            ssl_context.options |= ssl.OP_NO_TLSv1_1    # Python 3.4
-
-            ssl_context.verify_mode = ssl.CERT_REQUIRED     # https://docs.python.org/3.5/library/ssl.html#ssl.SSLContext.verify_mode
-            ssl_context.check_hostname = True               # https://docs.python.org/3.5/library/ssl.html#ssl.SSLContext.check_hostname
 
         if self.trust == TRUST_ALL_CERTIFICATES:
             ssl_context.check_hostname = False
             ssl_context.verify_mode = ssl.CERT_NONE     # https://docs.python.org/3.5/library/ssl.html#ssl.CERT_NONE
 
-        ssl_context.set_default_verify_paths()  # https://docs.python.org/3.5/library/ssl.html#ssl.SSLContext.set_default_verify_paths
+        # Must be load_default_certs, not set_default_verify_paths to work
+        # on Windows with system CAs.
+        ssl_context.load_default_certs()
+
         return ssl_context
 
 
@@ -269,6 +263,12 @@ class WorkspaceConfig(Config):
     """ WorkSpace configuration.
     """
 
+    #: Session Connection Timeout
+    session_connection_timeout = float("inf")  # seconds
+    # The maximum amount of time to wait for a session to obtain a usable
+    # read/write connection. This includes everything necessary for this to
+    # happen. Including fetching routing tables, opening sockets, etc.
+
     #: Connection Acquisition Timeout
     connection_acquisition_timeout = 60.0  # seconds
     # The maximum amount of time a session will wait when requesting a connection from the connection pool.
@@ -296,6 +296,10 @@ class WorkspaceConfig(Config):
     #: Fetch Size
     fetch_size = 1000
 
+    #: User to impersonate
+    impersonated_user = None
+    # Note that you need appropriate permissions to do so.
+
 
 class SessionConfig(WorkspaceConfig):
     """ Session configuration.
