@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- encoding: utf-8 -*-
 
-# Copyright (c) 2002-2020 "Neo4j,"
+# Copyright (c) "Neo4j"
 # Neo4j Sweden AB [http://neo4j.com]
 #
 # This file is part of Neo4j.
@@ -19,12 +19,15 @@
 # limitations under the License.
 
 
+from contextlib import contextmanager
+import socket
 from struct import pack as struct_pack
 
+from neo4j._exceptions import SocketDeadlineExceeded
 from neo4j.exceptions import (
     Neo4jError,
-    AuthError,
     ServiceUnavailable,
+    SessionExpired,
 )
 from neo4j.packstream import (
     UnpackableBuffer,
@@ -39,6 +42,7 @@ class MessageInbox:
 
     def __init__(self, s, on_error):
         self.on_error = on_error
+        self._local_port = s.getsockname()[1]
         self._messages = self._yield_messages(s)
 
     def _yield_messages(self, sock):
@@ -53,7 +57,7 @@ class MessageInbox:
                     buffer.receive(sock, 2)
                     chunk_size = buffer.pop_u16()
                     if chunk_size == 0:
-                        log.debug("[#%04X]  S: <NOOP>", sock.getsockname()[1])
+                        log.debug("[#%04X]  S: <NOOP>", self._local_port)
 
                 buffer.receive(sock, chunk_size + 2)
                 chunk_size = buffer.pop_u16()
@@ -66,7 +70,7 @@ class MessageInbox:
                     # Reset for new message
                     unpacker.reset()
 
-        except OSError as error:
+        except (OSError, socket.timeout, SocketDeadlineExceeded) as error:
             self.on_error(error)
 
     def pop(self):
@@ -87,54 +91,113 @@ class Inbox(MessageInbox):
 
 class Outbox:
 
-    def __init__(self, capacity=8192, max_chunk_size=16384):
+    def __init__(self, max_chunk_size=16384):
         self._max_chunk_size = max_chunk_size
-        self._header = 0
-        self._start = 2
-        self._end = 2
-        self._data = bytearray(capacity)
+        self._chunked_data = bytearray()
+        self._raw_data = bytearray()
+        self.write = self._raw_data.extend
+        self._tmp_buffering = 0
 
     def max_chunk_size(self):
         return self._max_chunk_size
 
     def clear(self):
-        self._header = 0
-        self._start = 2
-        self._end = 2
-        self._data[0:2] = b"\x00\x00"
+        if self._tmp_buffering:
+            raise RuntimeError("Cannot clear while buffering")
+        self._chunked_data = bytearray()
+        self._raw_data.clear()
 
-    def write(self, b):
-        to_write = len(b)
-        max_chunk_size = self._max_chunk_size
-        pos = 0
-        while to_write > 0:
-            chunk_size = self._end - self._start
-            remaining = max_chunk_size - chunk_size
-            if remaining == 0 or remaining < to_write <= max_chunk_size:
-                self.chunk()
-            else:
-                wrote = min(to_write, remaining)
-                new_end = self._end + wrote
-                self._data[self._end:new_end] = b[pos:pos+wrote]
-                self._end = new_end
-                pos += wrote
-                new_chunk_size = self._end - self._start
-                self._data[self._header:(self._header + 2)] = struct_pack(">H", new_chunk_size)
-                to_write -= wrote
+    def _chunk_data(self):
+        data_len = len(self._raw_data)
+        num_full_chunks, chunk_rest = divmod(
+            data_len, self._max_chunk_size
+        )
+        num_chunks = num_full_chunks + bool(chunk_rest)
 
-    def chunk(self):
-        self._header = self._end
-        self._start = self._header + 2
-        self._end = self._start
-        self._data[self._header:self._start] = b"\x00\x00"
+        data_view = memoryview(self._raw_data)
+        header_start = len(self._chunked_data)
+        data_start = header_start + 2
+        raw_data_start = 0
+        for i in range(num_chunks):
+            chunk_size = min(data_len - raw_data_start,
+                             self._max_chunk_size)
+            self._chunked_data[header_start:data_start] = struct_pack(
+                ">H", chunk_size
+            )
+            self._chunked_data[data_start:(data_start + chunk_size)] = \
+                data_view[raw_data_start:(raw_data_start + chunk_size)]
+            header_start += chunk_size + 2
+            data_start = header_start + 2
+            raw_data_start += chunk_size
+        del data_view
+        self._raw_data.clear()
+
+    def wrap_message(self):
+        if self._tmp_buffering:
+            raise RuntimeError("Cannot wrap message while buffering")
+        self._chunk_data()
+        self._chunked_data += b"\x00\x00"
 
     def view(self):
-        end = self._end
-        chunk_size = end - self._start
-        if chunk_size == 0:
-            return memoryview(self._data[:self._header])
+        if self._tmp_buffering:
+            raise RuntimeError("Cannot view while buffering")
+        self._chunk_data()
+        return memoryview(self._chunked_data)
+
+    @contextmanager
+    def tmp_buffer(self):
+        self._tmp_buffering += 1
+        old_len = len(self._raw_data)
+        try:
+            yield
+        except Exception:
+            del self._raw_data[old_len:]
+            raise
+        finally:
+            self._tmp_buffering -= 1
+
+
+class ConnectionErrorHandler:
+    """
+    Wrapper class for handling connection errors.
+
+    The class will wrap each method to invoke a callback if the method raises
+    Neo4jError, SessionExpired, or ServiceUnavailable.
+    The error will be re-raised after the callback.
+    """
+
+    def __init__(self, connection, on_error):
+        """
+        :param connection the connection object to warp
+        :type connection Bolt
+        :param on_error the function to be called when a method of
+            connection raises of of the caught errors.
+        :type on_error callable
+        """
+        self.__connection = connection
+        self.__on_error = on_error
+
+    def __getattr__(self, name):
+        connection_attr = getattr(self.__connection, name)
+        if not callable(connection_attr):
+            return connection_attr
+
+        def outer(func):
+            def inner(*args, **kwargs):
+                try:
+                    func(*args, **kwargs)
+                except (Neo4jError, ServiceUnavailable, SessionExpired) as exc:
+                    self.__on_error(exc)
+                    raise
+            return inner
+
+        return outer(connection_attr)
+
+    def __setattr__(self, name, value):
+        if name.startswith("_" + self.__class__.__name__ + "__"):
+            super().__setattr__(name, value)
         else:
-            return memoryview(self._data[:end])
+            setattr(self.__connection, name, value)
 
 
 class Response:
@@ -142,9 +205,10 @@ class Response:
     more detail messages followed by one summary message).
     """
 
-    def __init__(self, connection, **handlers):
+    def __init__(self, connection, message, **handlers):
         self.connection = connection
         self.handlers = handlers
+        self.message = message
         self.complete = False
 
     def on_records(self, records):
@@ -169,7 +233,10 @@ class Response:
     def on_failure(self, metadata):
         """ Called when a FAILURE message has been received.
         """
-        self.connection.reset()
+        try:
+            self.connection.reset()
+        except (SessionExpired, ServiceUnavailable):
+            pass
         handler = self.handlers.get("on_failure")
         if callable(handler):
             handler(metadata)
@@ -193,11 +260,12 @@ class InitResponse(Response):
 
     def on_failure(self, metadata):
         code = metadata.get("code")
-        message = metadata.get("message", "Connection initialisation failed")
         if code == "Neo.ClientError.Security.Unauthorized":
-            raise AuthError(message)
+            raise Neo4jError.hydrate(**metadata)
         else:
-            raise ServiceUnavailable(message)
+            raise ServiceUnavailable(
+                metadata.get("message", "Connection initialisation failed")
+            )
 
 
 class CommitResponse(Response):
